@@ -45,6 +45,7 @@
 #include <nlohmann/json.hpp>
 
 #include <dwarfs/file_stat.h>
+#include <dwarfs/fstypes.h>
 #include <dwarfs/reader/block_range.h>
 #include <dwarfs/reader/fsinfo_features.h>
 #include <dwarfs/reader/metadata_types.h>
@@ -60,6 +61,11 @@ class mmif;
 class os_access;
 class performance_monitor;
 
+namespace thrift::metadata {
+class fs_options;
+class metadata;
+} // namespace thrift::metadata
+
 namespace reader {
 
 struct cache_tidy_config;
@@ -96,6 +102,12 @@ class filesystem_v2_lite {
       filesystem_options const& options,
       std::shared_ptr<performance_monitor const> const& perfmon = nullptr);
 
+  filesystem_version version() const { return lite_->version(); }
+
+  bool has_valid_section_index() const {
+    return lite_->has_valid_section_index();
+  }
+
   void walk(std::function<void(dir_entry_view)> const& func) const {
     lite_->walk(func);
   }
@@ -316,6 +328,11 @@ class filesystem_v2_lite {
     return lite_->get_block_category(block_number);
   }
 
+  std::optional<nlohmann::json>
+  get_block_category_metadata(size_t block_number) const {
+    return lite_->get_block_category_metadata(block_number);
+  }
+
   void cache_blocks_by_category(std::string_view category) const {
     lite_->cache_blocks_by_category(category);
   }
@@ -330,6 +347,8 @@ class filesystem_v2_lite {
    public:
     virtual ~impl_lite() = default;
 
+    virtual filesystem_version version() const = 0;
+    virtual bool has_valid_section_index() const = 0;
     virtual void
     walk(std::function<void(dir_entry_view)> const& func) const = 0;
     virtual void
@@ -411,6 +430,8 @@ class filesystem_v2_lite {
     virtual std::vector<file_stat::gid_type> get_all_gids() const = 0;
     virtual std::optional<std::string>
     get_block_category(size_t block_number) const = 0;
+    virtual std::optional<nlohmann::json>
+    get_block_category_metadata(size_t block_number) const = 0;
     virtual void cache_blocks_by_category(std::string_view category) const = 0;
     virtual void cache_all_blocks() const = 0;
     virtual std::shared_ptr<internal::filesystem_parser> get_parser() const = 0;
@@ -473,6 +494,14 @@ class filesystem_v2 final : public filesystem_v2_lite {
 
   history const& get_history() const;
 
+  std::unique_ptr<thrift::metadata::metadata> thawed_metadata() const;
+  std::unique_ptr<thrift::metadata::metadata> unpacked_metadata() const;
+
+  std::unique_ptr<thrift::metadata::fs_options> thawed_fs_options() const;
+
+  std::future<block_range>
+  read_raw_block_data(size_t block_no, size_t offset, size_t size) const;
+
   class impl : public impl_lite {
    public:
     virtual int
@@ -484,6 +513,14 @@ class filesystem_v2 final : public filesystem_v2_lite {
     virtual std::string serialize_metadata_as_json(bool simple) const = 0;
     virtual std::optional<std::span<uint8_t const>> header() const = 0;
     virtual history const& get_history() const = 0;
+    virtual std::unique_ptr<thrift::metadata::metadata>
+    thawed_metadata() const = 0;
+    virtual std::unique_ptr<thrift::metadata::metadata>
+    unpacked_metadata() const = 0;
+    virtual std::unique_ptr<thrift::metadata::fs_options>
+    thawed_fs_options() const = 0;
+    virtual std::future<block_range>
+    read_raw_block_data(size_t block, size_t offset, size_t size) const = 0;
   };
 
  private:
